package ac.essex.gp.genetics;

import ac.essex.gp.params.GPParams;
import ac.essex.gp.tree.Node;
import ac.essex.gp.tree.TreeUtils;
import ac.essex.gp.genetics.selection.Selector;
import ac.essex.gp.genetics.selection.RandomNodeSelector;
import ac.essex.gp.genetics.selection.KozaNodeSelector;

/**
 * Performs crossover on two trees in a way that is compatible
 * with strong typing.
 *
 * @author Olly Oechsle, University of Essex, Date: 15-Jan-2007
 * @version 1.0
 */
public class StandardCrossover implements Crossover {

    private Selector s;
    
    public int maxDepth = 5;

    public StandardCrossover() {
        s = new KozaNodeSelector();
    }

    /**
     * How many parents does the crossover operator require?
     */
    public int getParentCount() {
        return 2;
    }

    public final Node[] produceOffspring(GPParams params, Node maleTree, Node femaleTree) {

        // get random subtree from the male individual
        //Node maleNode = TreeUtils.getRandomSubtree(maleTree, GPParams.ANY_RETURN_TYPE);
        Node maleNode = s.select(maleTree, GPParams.ANY_RETURN_TYPE);


        if (maleNode == null || maleNode.getTreeDepth() > maxDepth) {
            OperationCounter.FAILED_CROSSOVER_COUNT++;
            return null;
        }

        // get one from female, but ensure they match (in terms of return type AND size)
        //Node femaleNode = TreeUtils.getRandomSubtree(femaleTree, maleNode.getParentsExpectedType(params));
        Node femaleNode = s.select(femaleTree, maleNode.getParentsExpectedType(params));

        if (femaleNode == null || femaleNode.getTreeDepth() > maxDepth) {
            OperationCounter.FAILED_CROSSOVER_COUNT++;
            return null;
        }

        // save parents (replaceChild updates them)
        Node maleParent = maleNode.getParent();
        Node femaleParent = femaleNode.getParent();

        // elegant debugging
        if (maleParent == null) System.err.println("MALE PARENT IS NULL");
        if (femaleParent == null) System.err.println("FEMALE PARENT IS NULL");

        if (maleTree.getTreeSize() < 10 && femaleTree.getTreeSize() < 10) {
  /*          System.out.println("Male Tree");
            System.out.println(maleTree.toLisp());
            System.out.println("Female Tree");
            System.out.println(femaleTree.toLisp());
            System.out.println("Male Node");
            System.out.println(maleNode.toLisp());
            System.out.println("Female Node");
            System.out.println(femaleNode.toLisp());*/
        }

        // swap subtrees
        maleParent.replaceChild(maleNode, femaleNode);
        femaleParent.replaceChild(femaleNode, maleNode);

        OperationCounter.CROSSOVER_COUNT+=2;

if (maleTree.getTreeSize() < 10 && femaleTree.getTreeSize() < 10) {
/*                    System.out.println("Male Tree");
            System.out.println(maleTree.toLisp());
            System.out.println("Female Tree");
            System.out.println(femaleTree.toLisp());*/
}

        return new Node[]{maleTree,femaleTree};

    }




}
