package ac.essex.gp.nodes.imaging.features;

import ac.essex.ooechs.imaging.commons.texture.Circle;
import ac.essex.ooechs.imaging.commons.PixelLoader;
import ac.essex.gp.params.NodeConstraints;

/**
 * Returns the mean intensity of a region. See feature node for more info.
 *
 * @author Olly Oechsle, University of Essex, Date: 06-Nov-2007
 * @version 1.0
 */
public class CircleMean extends AbstractCircularNode {

    protected transient Circle c;

    private int dx, dy;

    public int[] getReturnTypes() {
        return new int[]{NodeConstraints.NUMBER, NodeConstraints.FLOATING_POINT_NUMBER, NodeConstraints.RANGE255, NodeConstraints.FEATURE};
    }

    public double getData(PixelLoader image, int windowLeft, int windowTop, int windowWidth, int windowHeight, int radius) {

        if (c == null) {
            c = new Circle(radius);
        }

        int centerX = windowWidth / 2;
        int centerY = windowHeight / 2;

        dx = (int) (centerX + (distance * Math.sin(angle)));
        dy = (int) (centerY + (distance * Math.cos(angle)));

        return c.getStatistics(image, windowLeft + dx, windowTop + dy).getMean();
    }

    public String toJava() {
        return "new Circle(" + radius + ").getStatistics(image, windowLeft + " + dx + ", windowTop + " + dy + ").getMean()";
    }

    public String getShortName() {
        return "Cm";
    }

}
