package ac.essex.gp.nodes.imaging.haar;

import ac.essex.gp.tree.Node;
import ac.essex.gp.params.NodeConstraints;
import ac.essex.gp.problems.DataStack;
import ac.essex.ooechs.imaging.commons.HaarRegions;

/**
 * <p/>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex, Date: 22-Apr-2008
 * @version 1.0
 */
public class ThreeRectHorizontalFeature extends Node {

    public ThreeRectHorizontalFeature() {
        super(4);
    }

    /**
     * Tells what return type each of the node's children should have
     */
    public int getChildType(int index) {
        return NodeConstraints.PARAMETER;
    }

    public int[] getReturnTypes() {
        return new int[]{NodeConstraints.NUMBER, NodeConstraints.FLOATING_POINT_NUMBER, NodeConstraints.RANGE255, NodeConstraints.FEATURE};
    }

    private int x, y, width, height;

    public double execute(DataStack data) {

        HaarRegions image = data.getHaar();

        double xp = child[0].execute(data);
        double yp = child[1].execute(data);
        double wp = child[2].execute(data);
        double hp = child[3].execute(data);

        if (xp + wp > 1) {
            wp = 1 - xp;
        }

        if (yp + hp > 1) {
            hp = 1 - yp;
        }

        x = (int) (image.getWindowBlocksX() * xp);
        y = (int) (image.getWindowBlocksY() * yp);
        width = (int) (image.getWindowBlocksX() * wp);
        height = (int) (image.getWindowBlocksY() * hp);

        data.value = data.getHaar().getThreeRectangleFeature(x, y, width, height, HaarRegions.HORIZONTALLY_ADJACENT);
        return debugger == null? data.value : debugger.record(data.value);
    }

    public String toJava() {
        return "image.getThreeRectangleFeature(" + x + ", " + y + ", " + width + ", " + height + ", HaarRegions.HORIZONTALLY_ADJACENT)";
    }

    public String getShortName() {
        return "3RH";
    }


}
