package ac.essex.gp.nodes.looping;

import ac.essex.gp.tree.Node;
import ac.essex.gp.problems.DataStack;
import ac.essex.gp.params.NodeConstraints;

/**
 * Performs one task after the other, then returns the final value
 *
 * @author Olly Oechsle, University of Essex, Date: 23-Oct-2008
 * @version 1.0
 */
public class List extends Node {

   public List(int numItems) {
       super(numItems);
   }

    /**
     * Any processing the node does takes place here
     */
    public double execute(DataStack data) {
        double value = 0;
        for (int i = 0; i < child.length; i++) {
            value = child[i].execute(data);
        }
        return value;
    }

    /**
     * Tells what return type each of the node's children should have
     *
     * @param index The index of the child, starting from 0
     */
    public int getChildType(int index) {
        return NodeConstraints.NUMBER;
    }

    public int[] getReturnTypes() {
        return new int[]{NodeConstraints.NUMBER, NodeConstraints.VOID};
    }

    /**
     * What would the java be if this node were written in it?
     */
    public String toJava() {
        return child[child.length-1].getName();
    }

    public Object[] getConstructorArgs() {
        // make sure the index is remembered when this node is cloned.
        return new Object[]{numChildren};
    }

}
